<?php

/*
  Plugin Name: Онлайн счетмаш
  Description: Плагин для управления кассой Онлайн-счетмаш.
  Author: KiisChivarino <kiischivarino@ya.ru>
  Version: 1.0
 */


new OnlineSchetmash;

class OnlineSchetmash {

    const SERVICE_URL = 'https://online.schetmash.com/lk/api/v1';
    const SERVICE_PORT = 443;

    /** @var array  */
    private static $lang = array(); // массив с переводом плагина

    /** @var string  */
    private static $pluginName = ''; // название плагина (соответствует названию папки)

    /** @var string  */
    private static $path = ''; //путь до файлов плагина

    /** @var array  */
    private  static $status = array();

    /** @var array  */
    private  static $types = array();

    /**
     * OnlineSchetmash constructor.
     */
    public function __construct() {
        self::$pluginName = PM::getFolderPlugin(__FILE__);
        self::$lang = PM::plugLocales(self::$pluginName);
        self::$path = PLUGIN_DIR.self::$pluginName;
        self::$status = array(
            'success' => self::$lang['STATUS_SUCCESS'],
            'no_conn' => self::$lang['STATUS_NO_CONN'],
            'accept' => self::$lang['STATUS_ACCEPT'],
            'fail' => self::$lang['STATUS_FAIL']
        );
        self::$types = array(
            'sell' => self::$lang['TYPE_SELL'],
            'sell_refund' => self::$lang['TYPE_SELL_REFUND'],
            'sell_correction' => self::$lang['TYPE_SELL_CORRECTION'],
            'buy' => self::$lang['TYPE_BUY'],
            'buy_refund' => self::$lang['TYPE_BUY_REFUND'],
            'buy_correction' => self::$lang['TYPE_BUY_CORRECTION'],
        );
        mgActivateThisPlugin(__FILE__, array(__CLASS__, 'activate')); //Инициализация  метода выполняющегося при активации
        mgAddAction(__FILE__, array(__CLASS__, 'pageSettingsPlugin')); //Инициализация  метода выполняющегося при нажатии на кнопку настроек плагина
        mgAddMeta('<script src="/mg-core/script/admin/orders.js"></script>');
        mgAddAction('Controllers_Payment_actionWhenPayment', array(__CLASS__, 'createPaymentCheck'),1);
        //mgAddAction('models_order_updateorder', array(__CLASS__, 'createPaymentCheck'),1);
        if(URL::isSection('schetmashresponse')) {
            mgAddAction('mg_gethtmlcontent', array(__CLASS__, 'schetmashResponse'), 1, 1);
        }
    }

    /**
     * Метод выполняющийся при активации палагина
     */
    static function activate() {
        $dropTableSql = 'DROP TABLE IF EXISTS `'.PREFIX.self::$pluginName.'`';
        $createTableSql = 'CREATE TABLE `'.PREFIX.self::$pluginName.'` (
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `status` varchar(20) NOT NULL,
            `uuid` int(11) DEFAULT NULL,
            `order_id` int(11) DEFAULT NULL,
            `data_create` datetime DEFAULT NULL,
            `data_print_end` datetime DEFAULT NULL,
            `sum` decimal(8,2) DEFAULT NULL,
            `type` varchar(20) DEFAULT NULL,
            `error` text,
            `payload` text,
            
            PRIMARY KEY (`id`),
            KEY `check_order_id` (`order_id`),
            KEY `check_data_create` (`data_create`),
            KEY `check_status` (`status`),
            KEY `check_type` (`type`)
          ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';
        //global $wpdb;
        DB::query($dropTableSql);

        DB::query($createTableSql);

        $array = Array(
            'width' => '980',
            'height' => '300',
            'countRows' => '10',
        );
        $hash = uniqid();
        setOption(array('option' => 'online-schetmashHash', 'value' => $hash));
        setOption(array('option' => 'online-schetmashOption', 'value' => addslashes(serialize($array))));
    }

    /**
     * Выводит страницу настроек плагина в админке
     */
    static function pageSettingsPlugin() {
        //получаем опцию schetmash-onlineOption в переменную option
        $options = unserialize(stripslashes(MG::getSetting('online-schetmashOption')));

        $lang = self::$lang;
        $pluginName = self::$pluginName;
        $res = self::getEntity(20);

        $entity = $res['entity'];
        $pagination = $res['pagination'];

        $status = self::$status;
        $types = self::$types;

      //begin для фрейма "настройки"

        // Статичный массив статусов.
        $statuses = array(
            0 => 'NOT_CONFIRMED',
            1 => 'EXPECTS_PAYMENT',
            2 => 'PAID',
            3 => 'IN_DELIVERY',
            4 => 'CANSELED',
            5 => 'EXECUTED',
            6 => 'PROCESSING',
        );

        $taxes = array(
            'vat20' => '20',
            'vat10' => '10',
            'vat0' => '0',
            'vat110' => '10/110' ,
            'vat120' =>	'20/120',
        );

        $payments = self::getPayment();

      //end для фрейма "настройки"

        self::preparePageSettings();
        include('pageplugin.php');
    }

    /**
     * Получает из БД записи
     */
    static function getEntity($count=1) {
        $result = array();
        $sql ="SELECT * FROM `".PREFIX.self::$pluginName."`";
        if ($_POST["page"]){
            $page = $_POST["page"]; //если был произведен запрос другой страницы, то присваиваем переменной новый индекс
        }
        $navigator = new Navigator($sql, $page, $count); //определяем класс
        $entity = $navigator->getRowsSql();
        $pagination = $navigator->getPager('forAjax');
        $result = array(
            'entity' => $entity,
            'pagination' => $pagination
        );
        return $result;
    }

    static function getPayment(){
        $paymentsArr = array();
        $res = DB::query('SELECT  *  FROM `'.PREFIX.'payment` ORDER BY `sort`');
        while ($row = DB::fetchAssoc($res)) {
            $paymentsArr[]=
                array(
                    'id'=>$row['id'],
                    'name'=>$row['name']
                );
        }
        return $paymentsArr;
    }

    /**
     * Метод выполняющийся перед генераццией страницы настроек плагина
     */
    static function preparePageSettings() {
        echo '   
      <link rel="stylesheet" href="'.SITE.'/'.self::$path.'/css/style.css" type="text/css" />     
      <script type="text/javascript">
        includeJS("'.SITE.'/'.self::$path.'/js/script.js");
        includeJS("'.SITE.'/'.self::$path.'/js/online-schetmash.js");
//        includeJS("'.SITE.'/'.self::$path.'/js/orders.js");
      </script> 
    ';
    }

    /**
     * Проверяет заказ на наличие
     * @param $id
     *
     * @return bool
     */
    static function checkOrder($id){
        $id = (int) $id;
        if($id) $res = DB::query('SELECT  *  FROM `'.PREFIX.'order` WHERE id='.$id);
        $answer = DB::numRows($res);
        return (bool) $answer;
    }

    /**
     * Добавляет чек автоматически при оплате
     * @param $arg
     */
    static function createPaymentCheck($arg){
        $order_id = $arg['args'][0]['id'];
        self::addCheck($order_id,'sell');
        //OnlineSchetmash::getPayment();
    }

    /**
     * Печатает чек
     * @param Models_Order $order
     * @param $operation
     *
     * @return bool|mixed
     */
    static function printCheck($order, $operation)
    {
        $options = unserialize(stripslashes(getOption('online-schetmashOption')));

        $checkQuery = new \stdClass;

        $date_created = $order['add_date'];
        $checkQuery->timestamp = $date_created;


        $dataUrl = URL::getDataUrl();
        $domain = $dataUrl['scheme'].'://'.$dataUrl['host'];

        $checkQuery->external_id = $operation . '_'. $domain . '_'.$order['id'];

        $checkQuery->service = new \stdClass;

        $api_token = getOption('schetmash_api_token');
        if(isset($api_token) && !empty($api_token)) {
            $api_token = '&api_token='.$api_token;
        } else {
            $api_token = '';
        }
        $hash = getOption('online-schetmashHash');
        $checkQuery->service->callback_url = PROTOCOL."://".$_SERVER['SERVER_NAME'].'/schetmashresponse'.$hash;
        $checkQuery->receipt = new \stdClass;
        $checkQuery->receipt->attributes = new \stdClass;

        $email = $order['user_email'];

        $phone = $order['phone'];

        if(isset($phone) && !empty($phone)){
            $phone = str_replace(array(' ','-','(',')'), '', $phone);
        }
        if(empty($phone) && empty($email)) {
            error_log(self::$lang['MESS_ERR_PHONE_MAIL']);
            return;
        }
        if($email) { $checkQuery->receipt->attributes->email = $email; }
        if($phone) { $checkQuery->receipt->attributes->phone = $phone; }
        if(empty($order['delivery_cost'])){
            $totalOrder = $order['summ'];
        }else{
            $totalOrder = $order['summ']*1 + $order['delivery_cost']*1;
        }
        $checkQuery->receipt->payments = array();
        $checkQuery->receipt->items = array();
        $checkQuery->receipt->total = self::formatFloat( $totalOrder );
        $checkQuery->receipt->payments[] = array(
            'type' => 1,
            'sum' =>  self::formatFloat( $totalOrder )
        );
        //todo добавить вычет скидки из суммы за товар или прибавить наценку или что-то сделать с промокодом
//        $totalDiscount = $order->get_total_discount();
//        $discount = 1;
//        if($totalDiscount > 0) {
//            $discount =  $totalOrder / ($totalOrder + $totalDiscount);
//        }

        $products = unserialize(stripslashes($order['order_content']));
        $tax = 'none';
        if(!empty($options['tax'])){
            $tax = $options['tax'];
        }

        $productsCount = count($products);
        if($productsCount==0){
            error_log('No products in order');
            return false;
        }

        if($order['delivery_cost']>0){
            $checkQuery->receipt->items[] = array(
                'name' => $order['description'],
                'price' => $order['delivery_cost'],
                'sum' => $order['delivery_cost'],
                'quantity' => 1,
                'tax' => $tax
            );
        }

        foreach ($products as $item)
        {

            $quantity = self::formatFloat( $item['count'], 3);

            $price = $item['fulPrice'];
            $sum = $item['fulPrice']*$quantity;

            $checkQuery->receipt->items[] = array(
                'name' => self::formatName( $item['name'] ),
                'price' => $price,
                'sum' =>  $sum,
                'quantity' =>  $quantity,
                'tax' => $tax,
            );
        }

        $token = $options['token'];
        $shopid = $options['shopid'];


        if(empty($shopid) ) {
            error_log(self::$lang['MESS_ERR_SHOP_ID']);
            return false;
        }

        $url = $shopid . '/' . $operation . '?token=';

        if (empty($token ))	{
            $token = self::getToken();
            if (!$token ) {	return false;	}
        }



        $result = self::send($url.$token, $checkQuery);

        if(!is_object($result) ) { return false;  }

        if (isset($result->code) && ($result->code  ==  2 || $result->code == 4) )	{
            $token = self::getToken();
            if (!$token)	{	return false;	}


            $result = self::send($url.$token, $checkQuery);
            if(!is_object($result) ) { return false;  }
        }


        if (isset($result->id)  )	{
            return $result;
        }

        return false;
    }

    /**
     * Подготавливает цены
     * @param $int
     * @param int $count
     *
     * @return float
     */
    static function formatFloat( $int, $count = 2){
        $int = str_replace(',', '.', $int);
        return	round( (float)$int , $count);
    }

    /**
     * Подготавливает заголовок чека
     * @param $name
     * @param int $count
     *
     * @return string|string[]|null
     */
    static function formatName($name, $count = 128){

        $name = preg_replace('|\s+|',' ', $name);
        $name = trim($name);
        $name = mb_substr ( $name, 0, $count, 'UTF-8');
        return $name;
    }

    /**
     * Получает токен
     * @return bool
     */
    static function getToken()
    {

        $options = unserialize(stripslashes(getOption('online-schetmashOption')));

        $data = new \stdClass;

        $data->login = (string)$options['bindlogin'];
        $data->password = (string)$options['bindpass'];
        if(empty($data->login) || empty($data->password) ) {
            error_log(self::$lang['MESS_ERR_LOGIN_PASS']);
            return false;
        }

        $result = self::send('token',  $data);

        if ( is_object($result) && isset($result->token) && !empty($result->token) )
        {
            $options['token'] = $result->token;
            setOption(array('option' => 'online-schetmashOption', 'value' => addslashes(serialize($options))));
            return $result->token;
        } else {
            error_log(self::$lang['MESS_ERR_TOKEN']);
        }

        if ( is_object($result) && isset($result->code)  && $result->code == 1 ) {
            error_log(self::$lang['MESS_ERR_TOKEN']);
        }

        return false;
    }

    /**
     * Отправляет чек на сайт online.schetmash.com
     * @param string $_path
     * @param null $_data
     *
     * @return bool|mixed
     */
    static function send($_path = '', $_data = null)    {

        //error_log(print_r($_data, 1));

        $curl = curl_init();

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_URL, static::SERVICE_URL.'/'.$_path);
        curl_setopt($curl, CURLOPT_PORT , static::SERVICE_PORT);
        if(isset($_data)) {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($_data));
        }
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        $out = curl_exec($curl);
        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $err = curl_error($curl);
        curl_close($curl);

        if (!$code) {  error_log(self::$lang['MESS_ERR_SERVER']. $err); return false;  }
        if ((int)$code !== 200) { error_log(self::$lang['MESS_ERR_SERVER_CODE'].$code);  return false;      }
        return json_decode( $out );
    }

    /**
     * Добавляет чек (обновляет чек со статусом "нет связи")
     * @param $order_id
     * @param $operation
     * @param bool $update
     *
     * @return array
     * @throws Exception
     */
    static function addCheck($order_id, $operation, $update = false) {
        $options = unserialize(stripslashes(MG::getSetting('online-schetmashOption')));
        $order_id = (int)$order_id;
        $report = array('mess' => '', 'status' => false );

        //begin проверка чека на наличие в таблице
        if(!$update && self::issetCheck($order_id, $operation)) {
            $report['mess'] = self::$lang['MESS_ERR_CHECK_ALREADY'];
            return $report;
        }
        //end проверка чека на наличие в таблице

        //begin поиск и получение заказа для чека
        $order = new Models_Order();
        $order = $order->getOrder('id='.$order_id);

        if(!is_array($order) || !isset($order[$order_id])){
            if($order === false || empty($order)) {
                error_log(self::$lang['MESS_ERR_ORDER_NOT_FOUND']);
                $report['mess'] = self::$lang['MESS_ERR_ORDER_NOT_FOUND'];
                return $report;
            }
        }
        $order = $order[$order_id];
        //end поиск и получение заказа для чека

        //begin проверка на способ оплаты
        $payment = $order['payment_id'];
        $paymentsMethod = $options['payments'];
        if(!is_array( $paymentsMethod)) { $paymentsMethod = array();  }

        if( !isset($payment) ||  empty($payment) ||  !in_array($payment, $paymentsMethod )) {
            $report['mess'] = self::$lang['MESS_ERR_CHOOSE_PAYMENT'];
            return $report;
        }
        //end проверка на способ оплаты


        //begin проверка опции когда печатать чек прихода
        $statusOrder = $order['status_id'];
        $checkinputOption = $options['checkinput'];
        if(!is_array( $checkinputOption)) { $checkinputOption = array();  }

        if( !isset($statusOrder) ||  empty($statusOrder) ||  !in_array($statusOrder, $checkinputOption )) {
            $report['mess'] = self::$lang['MESS_ERR_CHOOSE_STATUS'];
            return $report;
        }
        //end проверка опции когда печатать чек прихода
        //begin печать чека, формирование результата
        $result = self::printCheck($order, $operation);

        if($result === false || !is_object($result)) {
            $result = new stdClass;
            $result->status = 'no_conn';
        }
        //end печать чека, формирование результата

        //begin формирование данных для таблицы
        $data = array();
        $data['status'] = substr((string)$result->status, 0, 20) ? substr( (string)$result->status, 0, 20) : 'NULL';
        //if(isset($result->id) ) {
            $data['uuid'] =  !empty($result->id) ? $result->id : 'NULL';
        //}
        $datetime = new \DateTime();
        $curDate = $datetime->format('Y-m-d H:i:s');
        if(!$update) {
            $data['order_id'] = !empty($order_id) ? $order_id : 'NULL';
            $data['data_create'] = $curDate;
        }
        $data['sum'] =	isset($order['summ']) ? self::formatFloat($order['summ']) : 'NULL';
        $data['type'] = !empty($operation) ? $operation : 'NULL';
        $data['error'] = (!empty($result->error) ? json_encode($result->error) : '' );
        //end формирование данных для таблицы

        //begin добавление данных в таблицу
        if(!$update) {
            DB::query('
              INSERT INTO `'.PREFIX.self::$pluginName.'` (`status`, `uuid`, `order_id`, `data_create`, `sum`, `type`, `error`) 
              VALUES (
                "'.($data['status']).'",
                '.$data['uuid'].',
                '.$data['order_id'].',
                "'.$data['data_create'].'",
                '.$data['sum'].',
                "'.$data['type'].'",
                "'.$data['error'].'"
                )');
            $id = DB::insertId();
        } else {
            DB::query(
                'UPDATE `'.PREFIX.self::$pluginName.'` 
                    SET `status`="'.$data['status'].'", `uuid`='.$data['uuid'].',`sum`='.$data['sum'].',`type`="'.$data['type'].'",`error`="'.$data['error'].'"
                    WHERE `id`='.$update
            );
            $id = $update;
        }
        //end добавление данных в таблицу

        if($id) {
            $data = self::getCheck($id);
            $report = array('mess' => self::$lang['MESS_INFO_CHECK_ADD'], 'status' => true, 'data'=> $data );
            return $report;
        } else{
            $data = self::getCheck($id);
            $report = array('mess' => self::$lang['MESS_ERR_CHECK_NOT_ADD'], 'status' => false, 'data'=> $data);
            return $report;
        }

    }

    /**
     * Получает статус чека
     * @param $uuid
     *
     * @return array
     */
    static function getStatusCheck($uuid) {
        $options = unserialize(stripslashes(MG::getSetting('online-schetmashOption')));
        $report = array(
            'mess' => '',
            'status' => false
        );

        $token = (string)$options['token'];
        $shopid = (int)$options['shopid'];

        if (empty($shopid)) {
            error_log('Нет shopid');
            $report['mess'] = 'Нет shopid';
            return $report;
        }

        $url = $shopid.'/report/'.$uuid.'?token=';

        if (empty($token)) {
            $token = self::getToken();
            if (!$token) {
                $report['mess'] = 'Не удалось получить токен';
                return $report;
            }
        }

        $result = self::send($url.$token);

        if (!is_object($result)) {
            $report['mess'] = 'Нет связи с сервером';
            return $report;
        }

        if (isset($result->code) && ($result->code == 2 || $result->code == 4)) {
            $token = self::getToken();
            if (!$token) {
                $report['mess'] = 'Не удалось получить токен';
                return $report;
            }

            $result = self::send($url.$token);
            if (!is_object($result)) {
                $report['mess'] = 'Нет связи с сервером';
                return $report;
            }
        }

        if (isset($result->id)) {
            return self::updateCheck($result);
        }

        return $report;
    }

    /**
     * Обновляет чек
     * @param $result
     *
     * @return array
     * @throws Exception
     */
    static function updateCheck($result) {

        $report = array('mess' => 'Не удалось получить статус чека', 'status' => false );
        if(!is_object($result) || !isset($result->id) || !isset($result->status) || $result->status == 'accept' || $result->status == 'processing')   {
            return $report; }

        $data = array();
        $data['status'] = substr( (string)$result->status, 0, 20);
        if(isset($result->timestamp) ) {
            $datetime = new \DateTime($result->timestamp);
            $data['data_print_end'] =  $datetime->format('Y-m-d H:i:s');
        }
        if(isset($result->error) && !empty($result->error) ) {
            $data['error'] =  '"'.addslashes(json_encode($result->error)).'"';
        } else {
            $data['error'] =  'NULL';
        }
        $receipt_datetime = null;
        if(isset($result->payload) && !empty($result->payload) ) {
            if(isset($result->payload->receipt_datetime) && !empty($result->payload->receipt_datetime)){
                $datetime = new \DateTime($result->payload->receipt_datetime);
                $data['data_print_end'] =  $datetime->format('Y-m-d H:i:s');
            }else{
                $data['data_print_end'] = 'NULL';
            }
            $data['payload'] =  '"'.addslashes(json_encode($result->payload)).'"';
        }else {
            $data['payload'] =  'NULL';
        }
        //$upd = $wpdb->update($wpdb->prefix.'schetmash',  $data, array('uuid' => $result->id));
        $sql = 'UPDATE `'.PREFIX.self::$pluginName.'` SET 
                `status`="'.$data['status'].'",
                `error`='.$data['error'].',
                `payload`='.$data['payload'].',
                `data_print_end` = "'.$data['data_print_end'].'"
                    WHERE `uuid`='.$result->id;

        $upd = DB::query($sql);
        if($upd) {
            $report = array('mess' => 'Статус чека изменён', 'status' => true );
        } else {
            $report = array('mess' => 'Не удалось обновить чек', 'status' => false );
        }

        return $report;
    }

    /**
     * Проверяет чек по коду заказа и типу операции
     * @param $order_id
     * @param $type
     *
     * @return bool
     */
    static function issetCheck($order_id, $type) {
        $res = DB::query('SELECT * FROM `'.PREFIX.self::$pluginName.'`'.' WHERE `order_id` = '.$order_id.' AND `type` = "'.$type.'"');
        $answer = DB::numRows($res);
        return (bool) $answer;
    }

    /**
     * Для крона обновления статуса чека, ошибкок и инфо
     */
    static function schetmashResponse() {
        $hash = getOption('online-schetmashHash');
        if(URL::isSection('schetmashresponse'.$hash) && $_SERVER['REQUEST_METHOD'] === 'POST') {
            $result = json_decode(file_get_contents('php://input'), true);
            if ($result['id']) {
                $object= new stdClass();
                $result = self::array_to_obj($result,$object);
                $report = self::updateCheck($result);
                if($report['status']){
                    header("HTTP/1.1 200 OK");
                    header('Content-Type: application/json');
                    echo '{"status":"success"}';
                }else{
                    header("HTTP/1.1 500");
                }
                exit;
            }
        }else{
            header("HTTP/1.1 404");
        }
    }

    static function array_to_obj($array, &$obj)
    {
        foreach ($array as $key => $value)
        {
            if (is_array($value))
            {
                $obj->$key = new stdClass();
                self::array_to_obj($value, $obj->$key);
            }
            else
            {
                $obj->$key = $value;
            }
        }
        return $obj;
    }

    /**
     * Получает чек по id
     * @param $id
     *
     * @return array
     */
    static function getCheck($id){
        $sql ="SELECT * FROM `".PREFIX.self::$pluginName."` WHERE ";
        $where = array('id'=>$id);
        $res = DB::fetchAssoc(DB::buildQuery($sql,$where));
        return $res;
    }
}